#include <linux/input.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>

int main(int argc, char *argv[])
{
  struct input_event play;
  struct input_event stop;
  struct ff_effect effect;
  int fd;
  int gain = 10; /* 0-100 */
  char name[256] = "Unknown";
  int retval;

  if(argc < 2)
  {
    printf("usage: fftest /dev/input/eventX\n");
    return 0;
  }
  fd = open(argv[1], O_RDWR);
  
  if(ioctl(fd, EVIOCGNAME(sizeof(name)), name) < 0) {
    perror("evdev ioctl");
  }
  printf("%s: %s\n", argv[1], name);

  /* turn LED on */
  play.type = EV_LED;
  play.code = LED_MISC;
  play.value = 1;

  write(fd, (const void*) &play, sizeof(play));

  /* fill the effect structure */
  effect.type = FF_PERIODIC;
  effect.id = -1; /* generate new effect */
  effect.u.periodic.waveform = FF_SINE;
  effect.u.periodic.period = 6; /* in ms, 100ms = 10Hz, 20ms = 50Hz 5ms = 200Hz */
  effect.u.periodic.magnitude = 150; /* strength of the effect (0-255) */
  effect.u.periodic.envelope.attack_length = 0; /* duration of the effect (ms) */
  effect.u.periodic.envelope.fade_length = 3000; /* duration of the effect (ms) */

  /* upload the effect */
  retval = ioctl(fd, EVIOCSFF, &effect);
  if (retval < 0)
    printf("cannot upload the effect\n");

  printf("effect %d uploaded.\n", effect.id);

  /* Set gain */
  play.type = EV_FF;
  play.code = FF_GAIN;
  play.value = 0xFFFFUL * gain / 100;
  
  /* write(fd, (const void*) &play, sizeof(play)); */
  
  /* Play an effect */
  play.type = EV_FF;
  play.code = 0;
  play.value = 1;
  
  write(fd, (const void*) &play, sizeof(play));

  sleep(4);

  /* Play an effect (again) */
  play.type = EV_FF;
  play.code = 0;
  play.value = 1;
  
  write(fd, (const void*) &play, sizeof(play));

  sleep(1);


  usleep(200000);

  /* Stop an effect */
  play.type = EV_FF;
  play.code = 0;
  play.value = 0;
  
  write(fd, (const void*) &play, sizeof(play));
  
  /* Remove an effect */
  retval = ioctl(fd, EVIOCRMFF, effect.id);
  if (retval < 0)
    printf("cannot remove the effect %d\n", effect.id);

  /* turn LED off */
  play.type = EV_LED;
  play.code = LED_MISC;
  play.value = 0;

  write(fd, (const void*) &play, sizeof(play));

  return 0;
}
