/*
 * testlibusb.c
 *
 *  Test suite program
 */

#include <stdio.h>
#include <string.h>
#include <usb.h>
#include <string.h>
#include <asm/errno.h>

/* bind on this vendor/product */
#include "usbid.h"

#define MAX_USB_MICE 10
struct usb_device* usb_mouse[MAX_USB_MICE];
int miceFound = 0;

int verbose = 0;

void do_usbmouse(struct usb_device *dev) {
  usb_dev_handle *mouse_device = 0;
  int ret = 0;
  int i, j ;

  printf("Doing usbmouse\n");
  if(dev->descriptor.idVendor != USB_MOUSE_VENDOR ||
     dev->descriptor.idProduct != USB_MOUSE_PRODUCT) {
    printf("ERR: Wasn't passed a usbmouse!\n");
    return;
  }
  if (dev->descriptor.iSerialNumber) {
    printf("Processing usb mouse id ...\n");
  } else {
    printf("Curious, device has no serial number.\n");
  }
  mouse_device = usb_open(dev);
  if(mouse_device > 0) {
    char buffer[512];

    ret = usb_set_configuration(mouse_device, dev->config->bConfigurationValue);
    if(ret < 0) {
      printf("Failed to set config 1: %d EBUS: %d ENONM: %d\n", ret, EBUSY, ENOMEM);
      goto bail;
    }

    ret = usb_claim_interface(mouse_device, 0);
    if(ret < 0) {
      printf("Failed to claim interface 0: %d EBUS: %d ENONM: %d\n", ret, EBUSY, ENOMEM);
      goto bail;
    }
    i = 1000;
    while(i--) {
      ret = usb_interrupt_read(mouse_device, 1, buffer, 4, 500000);
      if(ret < 0) {
        printf("Failure usb_interrupt_read  %d '%s'\n", ret, strerror(-ret));
        goto bail;
      } 
      printf("%04x Read %d bytes: ", i, ret);
      for(j = 0; j < ret; j++) {
        printf("%02x ",
               ((int)buffer[j] & 0x0ff));
      }
      printf("\n");
    }

	  /*
          printf("Claiming %x\n", dev->config[0].interface[0].altsetting[0].endpoint[0].bEndpointAddress);
          ret = usb_claim_interface(mouse_device, dev->config[0].interface[0].altsetting[0].endpoint[0].bEndpointAddress);
	  if(ret < 0) {
            printf("Failed to claim interface: %d EBUS: %d ENONM: %d\n", ret, EBUSY, ENOMEM);
	  } else {
          ret = usb_interrupt_read(mouse_device, dev->config[0].interface[0].altsetting[0].endpoint[0].bEndpointAddress, buffer, sizeof(buffer), 5000);
	  if(ret < 0) {
		  printf("Failure usb_interrupt_read  %d\n", ret);
	  } else {
		  printf("Read %d bytes!\n", ret);
	  }
	  */

  bail:
    usb_close(mouse_device);
  }
}

void print_endpoint(struct usb_endpoint_descriptor *endpoint)
{
  printf("      bEndpointAddress: %02xh\n", endpoint->bEndpointAddress);
  printf("      bmAttributes:     %02xh\n", endpoint->bmAttributes);
  printf("      wMaxPacketSize:   %d\n", endpoint->wMaxPacketSize);
  printf("      bInterval:        %d\n", endpoint->bInterval);
  printf("      bRefresh:         %d\n", endpoint->bRefresh);
  printf("      bSynchAddress:    %d\n", endpoint->bSynchAddress);
}

void print_altsetting(struct usb_interface_descriptor *interface)
{
  int i;

  printf("    bInterfaceNumber:   %d\n", interface->bInterfaceNumber);
  printf("    bAlternateSetting:  %d\n", interface->bAlternateSetting);
  printf("    bNumEndpoints:      %d\n", interface->bNumEndpoints);
  printf("    bInterfaceClass:    %d\n", interface->bInterfaceClass);
  printf("    bInterfaceSubClass: %d\n", interface->bInterfaceSubClass);
  printf("    bInterfaceProtocol: %d\n", interface->bInterfaceProtocol);
  printf("    iInterface:         %d\n", interface->iInterface);

  for (i = 0; i < interface->bNumEndpoints; i++)
    print_endpoint(&interface->endpoint[i]);
}

void print_interface(struct usb_interface *interface)
{
  int i;

  for (i = 0; i < interface->num_altsetting; i++)
    print_altsetting(&interface->altsetting[i]);
}

void print_configuration(struct usb_config_descriptor *config)
{
  int i;

  printf("  wTotalLength:         %d\n", config->wTotalLength);
  printf("  bDescriptorType:      %d\n", config->bDescriptorType);
  printf("  bNumInterfaces:       %d\n", config->bNumInterfaces);
  printf("  bConfigurationValue:  %d\n", config->bConfigurationValue);
  printf("  iConfiguration:       %d\n", config->iConfiguration);
  printf("  bmAttributes:         %02xh\n", config->bmAttributes);
  printf("  MaxPower:             %d\n", config->MaxPower);

  for (i = 0; i < config->bNumInterfaces; i++)
    print_interface(&config->interface[i]);
}

void found_usb_mouse(struct usb_device *dev) {
  if(miceFound < MAX_USB_MICE) {
    usb_mouse[miceFound++] = dev;
    if(verbose) printf("Found usb mouse fob #%d\n", miceFound);
  }
}

int print_device(struct usb_device *dev, int level)
{
  usb_dev_handle *udev;
  char description[256];
  char string[256];
  int ret, i;

  udev = usb_open(dev);
  if (udev) {
    if (dev->descriptor.iManufacturer) {
      ret = usb_get_string_simple(udev, dev->descriptor.iManufacturer, string, sizeof(string));
      if (ret > 0) {
        snprintf(description, sizeof(description), "(%04X)",
                 dev->descriptor.idVendor);
        snprintf(description+strlen(description), sizeof(description)-strlen(description), "%s ~ ", string);
      } else {
        snprintf(description, sizeof(description), "%04X - ",
                 dev->descriptor.idVendor);
      }
    } else {
      snprintf(description, sizeof(description), "%04X - ",
               dev->descriptor.idVendor);
    }

    if (dev->descriptor.iProduct) {
      ret = usb_get_string_simple(udev, dev->descriptor.iProduct, string, sizeof(string));
      if (ret > 0) {
        snprintf(description + strlen(description), sizeof(description) -
                 strlen(description), "(%04X)", dev->descriptor.idProduct);
        snprintf(description + strlen(description), sizeof(description) -
                 strlen(description), "%s", string);
      } else
        snprintf(description + strlen(description), sizeof(description) -
                 strlen(description), "%04X", dev->descriptor.idProduct);
    } else
      snprintf(description + strlen(description), sizeof(description) -
               strlen(description), "%04X", dev->descriptor.idProduct);

  } else
    snprintf(description, sizeof(description), "%04X - %04X",
             dev->descriptor.idVendor, dev->descriptor.idProduct);

  printf("%.*sDev #%d: %s\n", level * 2, "                    ", dev->devnum,
         description);

  if (udev && verbose) {
    if (dev->descriptor.iSerialNumber) {
      ret = usb_get_string_simple(udev, dev->descriptor.iSerialNumber, string, sizeof(string));
      if (ret > 0)
        printf("%.*s  - Serial Number: %s\n", level * 2,
               "                    ", string);
    }
  }

  if (udev)
    usb_close(udev);

  if (verbose) {
    if (!dev->config) {
      printf("  Couldn't retrieve descriptors\n");
      return 0;
    }

    for (i = 0; i < dev->descriptor.bNumConfigurations; i++)
      print_configuration(&dev->config[i]);
  } else {
    for (i = 0; i < dev->num_children; i++) {
      print_device(dev->children[i], level + 1);
	if(dev->children[i]->descriptor.idVendor == USB_MOUSE_VENDOR &&
	   dev->children[i]->descriptor.idProduct == USB_MOUSE_PRODUCT) {
	  found_usb_mouse(dev->children[i]);
	}
    }
  }

  return 0;
}

int main(int argc, char *argv[])
{
  struct usb_bus *bus;
  int bus_changes = 0;
  int dev_changes = 0;
  int i;

  if (argc > 1 && !strcmp(argv[1], "-v"))
    verbose = 1;

  usb_init();

  bus_changes = usb_find_busses();
  if(verbose) printf("Bus changes: %d\n", bus_changes);
  dev_changes = usb_find_devices();
  if(verbose) printf("Dev changes: %d\n", dev_changes);

  for (bus = usb_busses; bus; bus = bus->next) {
    struct usb_device *dev;
    for (dev = bus->devices; dev; dev = dev->next) {
      print_device(dev, 0);
      if(dev->descriptor.idVendor == USB_MOUSE_VENDOR &&
         dev->descriptor.idProduct == USB_MOUSE_PRODUCT) {
        found_usb_mouse(dev);
      }
    }
  }

  for(i = 0; i < miceFound; i++) {
    do_usbmouse(usb_mouse[i]);
  }
  return 0;
}
